﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Xml.Linq;
using VideoShow.Data.DataServices;

namespace VideoShow
{
    public class Global : System.Web.HttpApplication
    {
        protected void Application_Start(object sender, EventArgs e)
        {
            string administratorName = UserServices.UserRoles.Administrator.ToString();
            SetupDefaultRoles();
            SetupDefaultRoleUser(administratorName, administratorName);
            UserServices userServices = new UserServices();
            userServices.AssignRole(administratorName, UserServices.UserRoles.Uploader);
        }

        /// <summary>
        /// Check SchemaVersion table for future upgrades
        /// </summary>
        private void CheckDatabaseVersion()
        {
        }

        /// <summary>
        /// Ensures all default roles have been set up
        /// </summary>
        private void SetupDefaultRoles()
        {
            string[] defaultRoles = System.Enum.GetNames(typeof(UserServices.UserRoles));
            foreach (string role in defaultRoles)
            {
                CreateRoleIfMissing(role);
            }
        }

        /// <summary>
        /// Creates the role if missing.
        /// </summary>
        /// <param name="role">The role.</param>
        private void CreateRoleIfMissing(string role)
        {
            if(!Roles.RoleExists(role))
            {
                Roles.CreateRole(role);
            }
        }

        /// <summary>
        /// Sets up a default role user given a username and role. 
        /// Applies a default password since this account is created automatically.
        /// </summary>
        /// <param name="username">The username.</param>
        /// <param name="role">The role.</param>
        private void SetupDefaultRoleUser(string username, string role)
        {
            string defaultPassword = ConfigurationManager.AppSettings["DefaultPassword"];
            if (Membership.GetUser(username) == null)
            {
                UserServices userServices = new UserServices();
                userServices.CreateUser(username, defaultPassword, string.Empty, 8);
                MembershipUser user = Membership.GetUser(username);
                if (user != null)
                {
                    Roles.AddUserToRole(username, role);
                }
            }
        }
    }
}