﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;
using VideoShow.Streaming.Services.Config;
using VideoShow.Data.DataServices;

namespace VideoShow.Streaming.Services
{
    public class VideoEncoderService
    {
        private string GetJobFileTemplate()
        {
            return Utils.GetConfigFile("JobFile.xml");
        }

        #region IVideoEncoderService Members

        public string MediaEncoderPath
        {   get; set; }

        public string GetTmpDir(int vid)
        {
            string tmpDir = Path.Combine(Path.GetTempPath(), "VideoShowEncoder");
            tmpDir = Path.Combine(tmpDir, vid.ToString() + ".Encoded");
            return tmpDir;
        }

        /// <summary>
        /// Encodes the specified source file.
        /// </summary>
        /// <param name="sourceFile">The source file.</param>
        /// <param name="videoId">The video id.</param>
        /// <param name="workingDirectory">The working directory.</param>
        /// <returns></returns>
        public string Encode(string sourceFile, int videoId, string workingDirectory)
        {
            string exeEncoder = "Encoder.exe";
            string tmpjobFile = Path.GetTempFileName();
            string tmpDir = GetTmpDir(videoId);

            try
            {
                // -- Create local dir to encode             
                if (!Directory.Exists(tmpDir))
                    Directory.CreateDirectory(tmpDir);

                // -- Loading My Template job file
                XmlSerializer xs = new XmlSerializer(typeof(JobFile));
                JobFile jobfile = null;
                using (Stream configStream = File.OpenRead(System.IO.Path.Combine(workingDirectory, "JobFile.xml")))
                {
                    jobfile = (JobFile)xs.Deserialize(configStream);
                    jobfile.Job[0].OutputDirectory = tmpDir;
                    jobfile.Job[0].MediaFiles[0].Source = sourceFile;
                }
                using (Stream configStream = File.OpenWrite(tmpjobFile))
                {
                    xs.Serialize(configStream, jobfile);
                }

                // -- Create process start info
                ProcessStartInfo psi = new ProcessStartInfo(exeEncoder);
                psi.Arguments = "/JobFile \"" + tmpjobFile + "\"" + " /Log On";
                psi.WorkingDirectory = MediaEncoderPath;

                //Hide MediaEncoder process window
                psi.WindowStyle = ProcessWindowStyle.Hidden;

                Process p = Process.Start(psi);

                Console.WriteLine("Encoding >" + psi.FileName + " " + psi.Arguments);

                p.WaitForExit();
            }
            catch (Exception ex)
            {
                //Write error message if a more specific error hasn't been logged.
                VideoServices.SetVideoStatusStatic(videoId, VideoServices.VideoStatus.Failed, ex.ToString(), false);
                tmpDir = string.Empty;
            }
            return tmpDir;
        }

        #endregion
    }
}
