﻿using System;
using System.Configuration;
using System.IO;
using System.Threading;
using VideoShow.Streaming.Services;
using VideoShow.Data.DataServices;

namespace VideoShow.Utility
{
    public class VideoProcessor
    {
        /// <summary>
        /// Processes a video - encodes, uploads to Silverlight Streaming, and updates the video metadata in the database.
        /// </summary>
        /// <param name="mediaFilename">The media filename.</param>
        /// <param name="filesetName">Name of the fileset.</param>
        /// <param name="videoId">The video id.</param>
        public static void Process(string mediaFilename, string filesetName, int videoId)
        {
            try
            {
                string workingDirectory = new System.IO.FileInfo(mediaFilename).DirectoryName;
                VideoEncoderService encoder = new VideoEncoderService();
                encoder.MediaEncoderPath = ConfigurationManager.AppSettings["MediaEncoderPath"];
                string encodedOutput = encoder.Encode(mediaFilename, videoId, workingDirectory);

                System.IO.FileInfo fileInfo = new System.IO.FileInfo(mediaFilename);
                string encodedVideoFilename = Path.ChangeExtension(Path.Combine(encodedOutput, fileInfo.Name), ".wmv");
                string generatedThumbnailFilename = Path.ChangeExtension(encodedVideoFilename, null) + "_Thumb.jpg";

                if (File.Exists(encodedVideoFilename))
                {
                    string thumbnailUrl = "/Silverlight/VideoGrid/images/thumbnail_placeholder.png";
                    if (File.Exists(generatedThumbnailFilename))
                    {
                        //Silverlight Streaming won't serve images to Silverlight Applications it doesn't host, 
                        //so we need to save the thumbnails locally.
                        string thumbnailDirectory = workingDirectory.Replace("_temp_upload", "Images\\Thumbnails");
                        string thumbnailFilename = Path.Combine(thumbnailDirectory, new FileInfo(generatedThumbnailFilename).Name);
                        File.Copy(generatedThumbnailFilename, thumbnailFilename, true);
                        thumbnailUrl = "/images/thumbnails/" + (new FileInfo(thumbnailFilename).Name);
                    }
                    SilverlightStreamingUpload.Upload(encodedVideoFilename, filesetName, videoId, workingDirectory, thumbnailUrl);
                }
                else
                {
                    VideoServices.SetVideoStatusStatic(videoId, VideoServices.VideoStatus.Failed, "Encoded video file was not found", false);
                }
            }
            catch(Exception ex)
            {
                //Write error message if a more specific error hasn't been logged.
                VideoServices.SetVideoStatusStatic(videoId, VideoServices.VideoStatus.Failed, ex.ToString(), false);
            }
        }

        /// <summary>
        /// Starts a process in a background thread.
        /// </summary>
        /// <param name="threadStart">ThreadStart info describes the process to be started.</param>
        public static void StartBackgroundThread(System.Threading.ThreadStart threadStart)
        {
            if (threadStart != null)
            {
                Thread thread = new Thread(threadStart);
                thread.IsBackground = true;
                thread.Start();
            }
        }
    }
}
